package supermarketcheckoutsystem;

import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

public class SupermarketRequirementsTest {

    private Supermarket supermarket;
    private Customer customer;
    private Account customerAccount;
    private Account supermarketAccount;
    private Bank bank;
    private POS pos;
    private CashRegister cashRegister;
    private TAS tas;
    private Card customerCard;
    private Item item1, item2;

    @BeforeEach
    public void setUp() throws Exception {
        // Initialize bank and accounts
        bank = new Bank();
        customerAccount = new Account(1000); // Customer starts with €1000
        supermarketAccount = new Account(0); // Supermarket starts with €0
        bank.addAccount(customerAccount);
        bank.addAccount(supermarketAccount);

        // Initialize POS, TAS, and Cash Register
        tas = new TAS(bank);
        pos = new POS(tas, supermarketAccount.getAccountNumber());
        cashRegister = new CashRegister(pos);

        // Initialize supermarket
        supermarket = new Supermarket(cashRegister);

        // Create customer with a normal plan and a bank card
        customerCard = new Card(1234);
        customerAccount.addCard(customerCard);
        customer = new Customer("John", "Doe", 10, 1, supermarket.discounts.get("normal"), customerCard, 1234);
        supermarket.addCustomer(customer);

        // Add items to supermarket inventory
        item1 = new Item("Apple", 2.0, 0.5, 0, 101, Category.Other);  // €2 per unit
        item2 = new Item("Milk", 3.0, 1.0, 0, 102, Category.Other);   // €3 per unit
        supermarket.addToInventory(item1);
        supermarket.addToInventory(item2);
    }

    // Requirement R1: Customers should be able to complete a purchase at checkout
    @Test
    public void testCustomerCheckout() throws Exception {
        supermarket.addItemToCustomer(customer, item1, 5); // 5 apples
        double initialBalance = customerAccount.getBalance();

        supermarket.customerCheckout(customer, true, false);

        double expectedPrice = 5 * 2.0; // 5 Apples * €2 each
        assertEquals(initialBalance - expectedPrice, customerAccount.getBalance());
    }

    // Requirement R2: Purchases consist of a set of bought items
    @Test
    public void testPurchaseContainsMultipleItems() throws Exception {
        supermarket.addItemToCustomer(customer, item1, 3);
        supermarket.addItemToCustomer(customer, item2, 2);

        assertEquals(3, customer.getCurrentPurchase().get(item1));
        assertEquals(2, customer.getCurrentPurchase().get(item2));
    }

    // Requirement R3: Item price should depend on its quantity
    @Test
    public void testItemPriceBasedOnQuantity() throws Exception {
        supermarket.addItemToCustomer(customer, item1, 3); // 3 Apples
        supermarket.addItemToCustomer(customer, item2, 2); // 2 Milk

        double initialBalance = customerAccount.getBalance();
        supermarket.customerCheckout(customer, true, false);

        double expectedPrice = (3 * 2.0) + (2 * 3.0); // 3*€2 + 2*€3
        assertEquals(initialBalance - expectedPrice, customerAccount.getBalance());
    }

    // Requirement R4: Customers must pay using a bank card
    @Test
    public void testPaymentWithCard() throws Exception {
        supermarket.addItemToCustomer(customer, item1, 4);
        double initialBalance = customerAccount.getBalance();

        supermarket.customerCheckout(customer, true, false);

        assertTrue(initialBalance > customerAccount.getBalance()); // Balance should decrease
    }

    // Requirement R5: Specific discount plans (Prime and Platinum) should apply
    @Test
    public void testDiscountApplication() throws Exception {
        customer.setCustomerPlan(supermarket.discounts.get("prime")); // Apply Prime (20% off)
        supermarket.addItemToCustomer(customer, item1, 5);
        
        double initialBalance = customerAccount.getBalance();
        supermarket.customerCheckout(customer, true, false);

        double expectedPrice = 5 * 2.0 * (1 - 0.2); // 5*€2 with 20% discount
        assertEquals(initialBalance - expectedPrice, customerAccount.getBalance());
    }

    // Requirement R5b: New discount plans should be modifiable and addable
    @Test
    public void testNewDiscountPlan() throws Exception {
        supermarket.addDiscount("gold", 0.25, 0.5, 100); // New 25% discount
        customer.setCustomerPlan(supermarket.discounts.get("gold"));

        supermarket.addItemToCustomer(customer, item1, 4);
        double initialBalance = customerAccount.getBalance();

        supermarket.customerCheckout(customer, true, false);

        double expectedPrice = 4 * 2.0 * (1 - 0.25); // 4*€2 with 25% discount
        assertEquals(initialBalance - expectedPrice, customerAccount.getBalance());
    }

    // Requirement R6: Apply specific pricing policies to item categories
    @Test
    public void testCategoryPricingPolicy() throws Exception {
        supermarket.setPricingPolicy(item1, 0.1); // 10% off all "Other" category
        supermarket.addItemToCustomer(customer, item1, 2);
        
        double initialBalance = customerAccount.getBalance();
        supermarket.customerCheckout(customer, true, false);

        double expectedPrice = 2 * 2.0 * (1 - 0.1); // 2 Apples * €2 * 10% off
        assertEquals(initialBalance - expectedPrice, customerAccount.getBalance());
    }

    // Requirement R6b: New pricing policies should be modifiable and addable
    @Test
    public void testNewPricingPolicy() throws Exception {
        supermarket.setPricingPolicy(item2, 0.2); // 20% off for Milk
        supermarket.addItemToCustomer(customer, item2, 3);

        double initialBalance = customerAccount.getBalance();
        supermarket.customerCheckout(customer, true, false);

        double expectedPrice = 3 * 3.0 * (1 - 0.2); // 3 Milk * €3 * 20% off
        assertEquals(initialBalance - expectedPrice, customerAccount.getBalance());
    }

    // Requirement R7: Customers should be able to choose delivery
    @Test
    public void testPurchaseWithDelivery() throws Exception {
        supermarket.addItemToCustomer(customer, item1, 3);
        double initialBalance = customerAccount.getBalance();

        supermarket.customerCheckout(customer, true, true); // Request delivery

        assertTrue(initialBalance > customerAccount.getBalance()); // Additional charge expected
    }

    // Requirement R8: Delivery should be charged based on weight & distance
    @Test
    public void testDeliveryCharge() throws Exception {
        supermarket.addItemToCustomer(customer, item1, 5); // 5 Apples (0.5kg each)
        double initialBalance = customerAccount.getBalance();

        supermarket.customerCheckout(customer, true, true);

        double itemCost = 5 * 2.0;
        double deliveryCost = 5 * 0.5 * Math.floor(customer.getDeliveryDistance()); // Based on weight & distance
        assertEquals(initialBalance - (itemCost + deliveryCost), customerAccount.getBalance());
    }

    // Requirement R8b: Prime and Platinum customers receive delivery discounts
    @Test
    public void testDeliveryDiscountForPrime() throws Exception {
        customer.setCustomerPlan(supermarket.discounts.get("prime")); // Prime (50% delivery discount)
        supermarket.addItemToCustomer(customer, item1, 4);
        double initialBalance = customerAccount.getBalance();

        supermarket.customerCheckout(customer, true, true);

        double itemCost = 4 * 2.0 * (1 - 0.2);
        double deliveryCost = (4 * 0.5 * Math.floor(customer.getDeliveryDistance())) * 0.5;
        assertEquals(initialBalance - (itemCost + deliveryCost), customerAccount.getBalance());
    }
}