package supermarketcheckoutsystem;

import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class SupermarketTest {
    private Supermarket supermarket;
    private CashRegister cashRegister;
    private Customer customer;
    private Item item;
    private Bank bank;
    private Account customerAccount;
    private Account supermarketAccount;
    private POS pos;
    private TAS tas;
    private Card customerCard;
    
    @BeforeEach
    void setUp() throws Exception {
        // Bank and Accounts
        bank = new Bank();
        customerAccount = new Account(100.0); // Customer starts with 100€
        supermarketAccount = new Account(1000.0); // Supermarket starts with 1000€
        bank.addAccount(customerAccount);
        bank.addAccount(supermarketAccount);
        
        // Initialize TAS, POS, and Cash Register
        tas = new TAS(bank);
        pos = new POS(tas, supermarketAccount.getAccountNumber());
        cashRegister = new CashRegister(pos);
        supermarket = new Supermarket(cashRegister);
        
        // Card setup
        customerCard = new Card(1234);
        customerCard.setAccountNumber(customerAccount.getAccountNumber());
        customerAccount.addCard(customerCard);
        
        // Customer setup
        customer = new Customer("John", "Doe", 10, 1, supermarket.discounts.get("normal"), customerCard, 1234);
        
        // Item setup
        item = new Item("Apple", 1.0, 0.2, 0, 1001, Category.Other);
    }

    // ----------- Supermarket Tests -----------
    
    @Test
    void testAddToInventory() {
        supermarket.addToInventory(item);
        assertTrue(supermarket.inventory.containsKey("Apple"));
    }

    @Test
    void testAddCustomer() {
        supermarket.addCustomer(customer);
        assertTrue(supermarket.customers.contains(customer));
    }

    @Test
    void testSetPricingPolicy() throws Exception {
        supermarket.addToInventory(item);
        supermarket.setPricingPolicy(item, 0.1);
        assertEquals(0.1, item.pricingPolicy);
    }

    @Test
    void testApplyDiscountPlan() throws Exception {
    	DiscountPlan primeDiscount = supermarket.discounts.get("prime");
        customer.setCustomerPlan(primeDiscount);
        supermarket.addCustomer(customer);
        supermarket.addToInventory(item);
        supermarket.addItemToCustomer(customer, item, 2);
        
        double expectedPrice = 2 * item.price * (1 - primeDiscount.discount);
        
        supermarket.customerCheckout(customer, true, false);
        
        assertEquals(100.0 - expectedPrice, customerAccount.getBalance(), 0.01);
    }

    // ----------- Bank Tests -----------

    @Test
    void testAccountBalanceUpdate() {
        customerAccount.updateBalance(-10);
        assertEquals(90.0, customerAccount.getBalance());
    }

    @Test
    void testCheckBalance() {
        assertTrue(bank.checkBalance(customerAccount.getAccountNumber(), 50));
        assertFalse(bank.checkBalance(customerAccount.getAccountNumber(), 200));
    }

    // ----------- TAS Tests -----------

    @Test
    void testTransactionAuthorization() {
        assertTrue(tas.authorisation(customerAccount.getAccountNumber(), 50));
        assertFalse(tas.authorisation(customerAccount.getAccountNumber(), 150));
    }

    @Test
    void testProcessTransaction() {
        boolean transactionResult = tas.processTransaction(customerAccount.getAccountNumber(), supermarketAccount.getAccountNumber(), 20);
        assertTrue(transactionResult);
        assertEquals(80.0, customerAccount.getBalance());
        assertEquals(1020.0, supermarketAccount.getBalance());
    }

    // ----------- POS Tests -----------

    @Test
    void testCardVerification() {
        assertTrue(pos.verifyPin(customerCard, customer));
    }

    @Test
    void testProcessPOSPayment() throws Exception {
        supermarket.addCustomer(customer);
        supermarket.addToInventory(item);
        supermarket.addItemToCustomer(customer, item, 2);
        
        double initialCustomerBalance = customerAccount.getBalance();
        supermarket.customerCheckout(customer, true, false);
        
        double expectedPrice = 2 * item.price;
        assertEquals(initialCustomerBalance - expectedPrice, customerAccount.getBalance());
    }

    // ----------- Cash Register Tests -----------

    @Test
    void testCustomerCheckout() throws Exception {
        supermarket.addCustomer(customer);
        supermarket.addToInventory(item);
        supermarket.addItemToCustomer(customer, item, 3);
        
        double initialCustomerBalance = customerAccount.getBalance();
        supermarket.customerCheckout(customer, true, false);
        
        double expectedPrice = 3 * item.price;
        assertEquals(initialCustomerBalance - expectedPrice, customerAccount.getBalance());
    }

    // ----------- Account and Card Tests -----------

    @Test
    void testAddCardToAccount() {
        assertTrue(customerAccount.getBalance() > 0);
        assertEquals(customerAccount.getAccountNumber(), customerCard.getAccountNumber());
    }

    @Test
    void testCheckCardPin() {
        assertTrue(customerCard.checkPin(1234));
        assertFalse(customerCard.checkPin(9999));
    }
}
